#!/usr/bin/perl

#
# Simple script that runs tosvcd
#
# by Thomas Koch <tom@linvdr.org>
#
# History:
# 20.07.2002 v0.1: initial release
# 30.08.2002 v0.2: dunno
# 20.10.2002 v0.3: add locking mechanism
#                  add silent mode
# 16.12.2002 v0.4: add multiple convert methods
# 24.09.2003 v0.5: now using vdrsync
#

use Getopt::Std;
use strict;

my $TODO_LIST       = "/etc/vdr/toconvert.txt";
my $LOCK_FILE       = "/etc/vdr/toconvert_lock";
my $OUTPUT          = "/pub/export";
my $VDRSYNC         = "/usr/bin/vdrsync-orig.pl";
my $NICE            = "/usr/bin/nice -n 19";
my $MKISOFS         = "/usr/bin/mkisofs";
my $NOAD            = "/usr/bin/noad";
my $MODE            = 0;
my $SILENT          = 0;
my $DEBUG           = 0;
my $VERSION         = "0.6";

$SIG{INT} = \&RemoveLock;

my @MODE;
$MODE[0]   = "DVD";
$MODE[1]   = "MPEG2";
$MODE[2]   = "DVD Strukur";
$MODE[3]   = "Audio";
$MODE[255] = "noad";

sub Check {
  my $ok = 1;
  sub PrintError {
    print shift() . "\n";
    $ok = 0;
  }
  PrintError("$VDRSYNC is not executable.\n") if(!-x "$VDRSYNC");
  PrintError("$OUTPUT is not writable.\n") if(!-w "$OUTPUT");
  #PrintError("$TODO_LIST does not exist.\n") if(!-e "$TODO_LIST");
  #PrintError("$TODO_LIST is not writable.\n") if(!-w "$TODO_LIST");
  #PrintError("$TODO_LIST is not readable.\n") if(!-r "$TODO_LIST");
  
  exit(1) if(!$ok);
}

sub Warning {
  my $msg = join("", @_);
  print $msg;
}

sub getNextJob {
  if(-e $TODO_LIST) {
		open(FILE, $TODO_LIST) || Warning "cant read $TODO_LIST ($!)";
		$_ = <FILE>;
		close(FILE);
		chomp;
		$_ =~ /(\d+)\: (.*)/;
		return($1, $2);
	}
}

sub RemoveJob {
  my $toremove = shift;
  my @letin;
  open(INFILE, $TODO_LIST) || Warning "cant read $TODO_LIST ($!)";
  while(<INFILE>) {
    chomp;
    push(@letin, "$_\n") if($_ ne $toremove);
  }
  close INFILE;
   
  open(OUTFILE, ">$TODO_LIST") || die "cant write $TODO_LIST ($!)";
  print OUTFILE @letin;
  close OUTFILE;
}


sub List {
  if(-s $TODO_LIST) {
    open(FILE, $TODO_LIST) || die "cant read $TODO_LIST ($!)";
    while(<FILE>) {
      $_ =~ /(\d+)\: \/video0\/(.*)\/(\d\d\d\d)-(\d\d)-(\d\d)\.(\d\d)[\:\.](\d\d).\d\d\.\d\d.rec/;
      print sprintf("%s.%s.%s %s\n%s\n\n",
       $5,
       $4,
       $3,
       $MODE[$1],
       ListTranslate($2),
     );
    }
    close(FILE);
  } else {
    print "leer";
  }
}      

sub ListTranslate {
	$_ = shift;
	s/_/ /g;
	return($_);
}

sub Translate {
  $_ = shift;
  s//ae/g;
  s//oe/g;
  s//ue/g;
  return($_);
}

sub getOutputName {
  $_ = shift;
  /.*\/(.*)\/(\d\d\d\d)-(\d\d)-(\d\d)\.\d\d[\:\.]\d\d.\d\d\.\d\d.rec/;
  my $tmp = $1;
  $tmp =~ s/^\%//;
  $tmp = Translate($tmp);
  return $tmp;
}

sub Exec {
  my $cmd = shift;

  print "$cmd\n" if $DEBUG;
  my $return = system($cmd);                                                                                                            

  if($return) {                                                                                                                         
    print "Error while running \"$cmd\". Recording has been removed from $TODO_LIST!\n";                                                
    return 1;
  }
  return 0;
}

sub Transcode {
  while((my($mode, $dir) = getNextJob())) {
    if($dir) {
      if(!-d "$dir") {
        print "$dir does not exist. Recording has been removed from $TODO_LIST!\n";
        RemoveJob("$mode: $dir");
        next;
      }
      my $cmd;
      if($mode == 0) { # DVD Image
        $cmd = sprintf("%s %s -basename \"%s\" -o \"%s\" -mkiso \"%s\" %s", 
          $NICE, 
          $VDRSYNC, 
          getOutputName($dir),
          $OUTPUT,
          $dir, 
          $DEBUG ? "" : ">/dev/null 2>/dev/null");
        Exec($cmd);
      }
      if($mode == 1) { # MPEG2
        $cmd = sprintf("%s %s -basename \"%s\" -o \"%s\" -mpeg2 \"%s\" %s", 
          $NICE, 
          $VDRSYNC, 
          getOutputName($dir),
					$OUTPUT,
          $dir, 
          $DEBUG ? "" : ">/dev/null 2>/dev/null");
        Exec($cmd);
      }
      if($mode == 2) { # DVD Struktur
        $cmd = sprintf("%s %s -basename \"%s\" -o \"%s\" -master-dvd \"%s\" %s", 
          $NICE, 
          $VDRSYNC, 
          getOutputName($dir),
          $OUTPUT,
          $dir, 
          $DEBUG ? "" : ">/dev/null 2>/dev/null");
        Exec($cmd);
      }
      if($mode == 3) { # Audio
        $cmd = sprintf("%s %s -basename \"%s\" -o \"%s\" -audio-only \"%s\" %s", 
          $NICE, 
          $VDRSYNC, 
          getOutputName($dir),
          $OUTPUT,
          $dir, 
          $DEBUG ? "" : ">/dev/null 2>/dev/null");
        Exec($cmd);
      }
      if($mode == 255) {
        $cmd = sprintf("%s %s - \"%s\" %s",
          $NICE,
          $NOAD,
          $dir,
          $DEBUG ? "" : ">/dev/null 2>/dev/null");
        Exec($cmd);
      }
      
      #for my $ext (("iso", "mpeg", "mpeg")) {
#  	system(sprintf("... %s/*.%s 2>/dev/null >/dev/null", $OUTPUT, $ext));      
      #}
      system(sprintf("chown -R linvdr.root %s", $OUTPUT));
      RemoveJob("$mode: $dir");
    } else {
      return;
    }
  }
}

sub Lock {
  if(-e $LOCK_FILE) {
    return(1);
  } else {
    return(0);
  }
}

sub CreateLock {
  open(FILE, ">$LOCK_FILE") || die "can't write $LOCK_FILE ($!)\n";
  print FILE $$;
  close(FILE);
}

sub RemoveLock {
  unlink($LOCK_FILE);
  #unlink($LOCK_FILE) || die "can't remove $LOCK_FILE ($!)\n";
}

my %args;
getopts("dlshqm:r:", \%args);
if($args{l}) {
  List();
  exit;
}
if($args{d}) {
  $DEBUG = 1;
}
if($args{s}) {
  $SILENT = 1;
} 
if($args{m}) {
  $MODE = $args{m};
} 
if($args{q}) {
  Check();
  if(!Lock()) {
    CreateLock();
    Transcode();
    RemoveLock();
  } else {
    if(!$SILENT) {
      print "There's already an copy of this program running!\n";
      print "If you feel this is a error, remove $LOCK_FILE\n";
    }
    exit 1;
  }
  exit 0;
} 
if($args{r}) {
  open(FILE, ">>$TODO_LIST") || die "cant write $TODO_LIST ($!)";
  print FILE "$MODE: " . $args{r} . "\n";
  close FILE;
  print("Zur Liste hinzugefgt.");
  exit 0;
} 
if(1) { 
  printf("Usage %s [OPTION]...\n\n", join("", $0 =~ /^.*\/(.*)/));
  print("  -q  run queue\n");
  print("  -d  debug mode\n");
  print("  -s  silent mode, only errors are printed\n");
  print("  -m  which mode to run\n");
  print("  -l  list queue\n");
  print("  -h  you are reading this\n\n");
  print("Report bugs to <linvdr\@linvdr.org>\n");
  exit 0;
}

# EOF
